@@ -25,6 +25,7 @@ void determine_path_to_current_executable(std::string const &argv0);
 bfs::path get_current_exe_path(std::string const &argv0);
 bfs::path get_application_data_folder();
 bfs::path get_installation_path();
+uint64_t get_memory_usage();
 
 bool is_installed();
 
@@ -44,6 +45,7 @@ void set_environment_variable(const std::string &key, const std::string &value);
 #define WINDOWS_VERSION_7            0x00060001
 
 unsigned int get_windows_version();
+std::string format_windows_message(uint64_t message_id);
 
 #endif
 
