@@ -31,8 +31,8 @@
 namespace mpeg4 {
 namespace p10 {
 
-static auto s_debug_fix_bistream_timing_info = debugging_option_c{"fix_bitstream_timing_info"};
-static auto s_debug_remove_bistream_ar_info  = debugging_option_c{"remove_bitstream_ar_info"};
+static auto s_debug_fix_bistream_timing_info = debugging_option_c{"avc_parser|fix_bitstream_timing_info"};
+static auto s_debug_remove_bistream_ar_info  = debugging_option_c{"avc_parser|remove_bitstream_ar_info"};
 
 avcc_c::avcc_c()
   : m_profile_idc{}
@@ -265,6 +265,14 @@ mpeg4::p10::timing_info_t::default_duration()
   return 1000000000ll * num_units_in_tick / time_scale;
 }
 
+bool
+mpeg4::p10::timing_info_t::is_valid()
+  const {
+  return is_present
+      && (0 != num_units_in_tick)
+      && (0 != time_scale);
+}
+
 void
 mpeg4::p10::sps_info_t::dump() {
   mxinfo(boost::format("sps_info dump:\n"
@@ -327,9 +335,7 @@ mpeg4::p10::sps_info_t::dump() {
 bool
 mpeg4::p10::sps_info_t::timing_info_valid()
   const {
-  return timing_info.is_present
-      && (0 != timing_info.num_units_in_tick)
-      && (0 != timing_info.time_scale);
+  return timing_info.is_valid();
 }
 
 void
@@ -413,7 +419,7 @@ mpeg4::p10::parse_sps(memory_cptr const &buffer,
   else {
     auto frame_rate = mtx::frame_timing::determine_frame_rate(duration);
     if (!frame_rate)
-      frame_rate.assign(0x80000000, (duration * 0x80000000) / 1000000000ll);
+      frame_rate.assign(static_cast<int64_t>(0x80000000), static_cast<int64_t>((duration * 0x80000000) / 1000000000ll));
 
     num_units_in_tick = frame_rate.denominator();
     time_scale        = frame_rate.numerator();
@@ -563,6 +569,15 @@ mpeg4::p10::parse_sps(memory_cptr const &buffer,
       sps.timing_info.num_units_in_tick = r.get_bits(32);
       sps.timing_info.time_scale        = r.get_bits(32);
       sps.timing_info.fixed_frame_rate  = r.get_bit();
+
+      if (   sps.timing_info.is_valid()
+          && (sps.timing_info.default_duration() < 5'000ll)) {
+        mxdebug_if(s_debug_fix_bistream_timing_info,
+                   boost::format("timing info present && bogus values detected (#units %2% time_scale %3%); defaulting to 25 FPS (default duration 20'000'000)\n")
+                   % sps.timing_info.is_present % sps.timing_info.num_units_in_tick % sps.timing_info.time_scale);
+        sps.timing_info.num_units_in_tick = 1;
+        sps.timing_info.time_scale        = 50;
+      }
     }
 
     mxdebug_if(s_debug_fix_bistream_timing_info,
@@ -863,12 +878,12 @@ mpeg4::p10::avc_es_parser_c::avc_es_parser_c()
   , m_discard_actual_frames(false)
   , m_simple_picture_order{}
   , m_first_cleanup{true}
+  , m_all_i_slices_are_key_frames{hack_engaged(ENGAGE_ALL_I_SLICES_ARE_KEY_FRAMES)}
   , m_debug_keyframe_detection{"avc_parser|avc_keyframe_detection"}
   , m_debug_nalu_types{        "avc_parser|avc_nalu_types"}
   , m_debug_timecodes{         "avc_parser|avc_timecodes"}
   , m_debug_sps_info{          "avc_parser|avc_sps|avc_sps_info"}
-  , m_debug_trailing_zero_byte_removal{"avc_parser|avc_trailing_zero_byte_removal"}
-  , m_debug_sps_pps_changes{           "avc_parser|avc_sps_pps_changes"}
+  , m_debug_sps_pps_changes{   "avc_parser|avc_sps_pps_changes"}
 {
   if (m_debug_nalu_types)
     init_nalu_names();
@@ -912,30 +927,6 @@ mpeg4::p10::avc_es_parser_c::discard_actual_frames(bool discard) {
   m_discard_actual_frames = discard;
 }
 
-void
-mpeg4::p10::avc_es_parser_c::remove_trailing_zero_bytes(memory_c &memory) {
-  auto size = memory.get_size();
-
-  if (size < 3)
-    return;
-
-  auto start = memory.get_buffer();
-  auto end   = start + size - 3;
-
-  if (get_uint24_be(end) != 0x000000)
-    return;
-
-  --end;
-
-  while ((end > start) && (!*end))
-    --end;
-
-  auto new_size = end - start + 1;
-  memory.set_size(new_size);
-
-  mxdebug_if(m_debug_trailing_zero_byte_removal, boost::format("Removing trailing zero bytes from old size %1% down to new size %2%, removed %3%\n") % size % new_size % (size - new_size));
-}
-
 void
 mpeg4::p10::avc_es_parser_c::add_bytes(unsigned char *buffer,
                                        size_t size) {
@@ -967,8 +958,10 @@ mpeg4::p10::avc_es_parser_c::add_bytes(unsigned char *buffer,
           auto nalu = memory_c::alloc(new_size);
           cursor.copy(nalu->get_buffer(), previous_pos + previous_marker_size, new_size);
           m_parsed_position = previous_parsed_pos + previous_pos;
-          remove_trailing_zero_bytes(*nalu);
-          handle_nalu(nalu);
+
+          mtx::mpeg::remove_trailing_zero_bytes(*nalu);
+          if (nalu->get_size())
+            handle_nalu(nalu, m_parsed_position);
         }
         previous_pos         = cursor.get_position() - marker_size;
         previous_marker_size = marker_size;
@@ -1003,7 +996,8 @@ mpeg4::p10::avc_es_parser_c::flush() {
   if (m_unparsed_buffer && (5 <= m_unparsed_buffer->get_size())) {
     m_parsed_position += m_unparsed_buffer->get_size();
     int marker_size = get_uint32_be(m_unparsed_buffer->get_buffer()) == NALU_START_CODE ? 4 : 3;
-    handle_nalu(memory_c::clone(m_unparsed_buffer->get_buffer() + marker_size, m_unparsed_buffer->get_size() - marker_size));
+    auto nalu_size  = m_unparsed_buffer->get_size() - marker_size;
+    handle_nalu(memory_c::clone(m_unparsed_buffer->get_buffer() + marker_size, nalu_size), m_parsed_position - nalu_size);
   }
 
   m_unparsed_buffer.reset();
@@ -1017,8 +1011,7 @@ mpeg4::p10::avc_es_parser_c::flush() {
 
 void
 mpeg4::p10::avc_es_parser_c::add_timecode(int64_t timecode) {
-  m_provided_timecodes.push_back(timecode);
-  m_provided_stream_positions.push_back(m_stream_position);
+  m_provided_timestamps.emplace_back(timecode, m_stream_position);
   ++m_stats.num_timecodes_in;
 }
 
@@ -1072,12 +1065,8 @@ mpeg4::p10::avc_es_parser_c::flush_incomplete_frame() {
 
 void
 mpeg4::p10::avc_es_parser_c::flush_unhandled_nalus() {
-  std::deque<memory_cptr>::iterator nalu = m_unhandled_nalus.begin();
-
-  while (m_unhandled_nalus.end() != nalu) {
-    handle_nalu(*nalu);
-    nalu++;
-  }
+  for (auto const &nalu_with_pos : m_unhandled_nalus)
+    handle_nalu(nalu_with_pos.first, nalu_with_pos.second);
 
   m_unhandled_nalus.clear();
 }
@@ -1099,9 +1088,10 @@ mpeg4::p10::avc_es_parser_c::add_sps_and_pps_to_extra_data() {
 }
 
 void
-mpeg4::p10::avc_es_parser_c::handle_slice_nalu(memory_cptr const &nalu) {
+mpeg4::p10::avc_es_parser_c::handle_slice_nalu(memory_cptr const &nalu,
+                                               uint64_t nalu_pos) {
   if (!m_avcc_ready) {
-    m_unhandled_nalus.push_back(nalu);
+    m_unhandled_nalus.emplace_back(nalu, nalu_pos);
     return;
   }
 
@@ -1135,8 +1125,11 @@ mpeg4::p10::avc_es_parser_c::handle_slice_nalu(memory_cptr const &nalu) {
   m_incomplete_frame.m_si       =  si;
   m_incomplete_frame.m_keyframe =  m_recovery_point_valid
                                 || (   is_i_slice
-                                    && (NALU_TYPE_IDR_SLICE == si.nalu_type));
+                                    && (NALU_TYPE_IDR_SLICE == si.nalu_type))
+                                || (   is_i_slice
+                                    && m_all_i_slices_are_key_frames);
   m_incomplete_frame.m_type     =  m_incomplete_frame.m_keyframe ? 'I' : is_b_slice ? 'B' : 'P';
+  m_incomplete_frame.m_position =  nalu_pos;
   m_recovery_point_valid        =  false;
 
   if (m_incomplete_frame.m_keyframe) {
@@ -1146,6 +1139,13 @@ mpeg4::p10::avc_es_parser_c::handle_slice_nalu(memory_cptr const &nalu) {
                % (!si.field_pic_flag                ? "frame" : si.bottom_field_flag              ? "bottom field" : "top field")
                % (!m_current_key_frame_bottom_field ? "none"  : *m_current_key_frame_bottom_field ? "bottom field" : "top field"));
 
+    // if (!m_first_keyframe_found) {
+    //   mxinfo(boost::format("first KF; num prov TC %1% last prov TC %2%\n") % m_provided_timestamps.size() % format_timestamp(m_provided_timestamps.empty() ? -1 : m_provided_timestamps.back().first));
+    //   for (auto const &t : m_provided_timestamps)
+    //     mxinfo(boost::format("  %1% @ %2%\n") % format_timestamp(t.first) % t.second);
+
+    // }
+
     m_first_keyframe_found    = true;
     m_b_frames_since_keyframe = false;
     if (!si.field_pic_flag || !m_current_key_frame_bottom_field || (*m_current_key_frame_bottom_field == si.bottom_field_flag)) {
@@ -1170,11 +1170,6 @@ mpeg4::p10::avc_es_parser_c::handle_slice_nalu(memory_cptr const &nalu) {
   m_incomplete_frame.m_data = create_nalu_with_size(nalu, true);
   m_have_incomplete_frame   = true;
 
-  if (!m_provided_stream_positions.empty() && (m_parsed_position >= m_provided_stream_positions.front())) {
-    m_incomplete_frame.m_has_provided_timecode = true;
-    m_provided_stream_positions.pop_front();
-  }
-
   ++m_frame_number;
 }
 
@@ -1297,13 +1292,14 @@ mpeg4::p10::avc_es_parser_c::handle_sei_nalu(memory_cptr const &nalu) {
 }
 
 void
-mpeg4::p10::avc_es_parser_c::handle_nalu(memory_cptr const &nalu) {
+mpeg4::p10::avc_es_parser_c::handle_nalu(memory_cptr const &nalu,
+                                         uint64_t nalu_pos) {
   if (1 > nalu->get_size())
     return;
 
   int type = *(nalu->get_buffer()) & 0x1f;
 
-  mxdebug_if(m_debug_nalu_types, boost::format("NALU type 0x%|1$02x| (%2%) size %3%\n") % type % get_nalu_type_name(type) % nalu->get_size());
+  mxdebug_if(m_debug_nalu_types, boost::format("NALU type 0x%|1$02x| (%2%) at %3% size %4%\n") % type % get_nalu_type_name(type) % nalu_pos % nalu->get_size());
 
   switch (type) {
     case NALU_TYPE_SEQ_PARAM:
@@ -1335,7 +1331,7 @@ mpeg4::p10::avc_es_parser_c::handle_nalu(memory_cptr const &nalu) {
         m_avcc_ready = true;
         flush_unhandled_nalus();
       }
-      handle_slice_nalu(nalu);
+      handle_slice_nalu(nalu, nalu_pos);
       break;
 
     default:
@@ -1499,6 +1495,18 @@ mpeg4::p10::avc_es_parser_c::calculate_frame_order() {
   auto prev_pic_order_cnt_lsb = 0u;
   auto pic_order_cnt_msb      = 0u;
 
+  m_simple_picture_order      = false;
+
+  if (   (   (AVC_SLICE_TYPE_I   != idr.type)
+          && (AVC_SLICE_TYPE_SI  != idr.type)
+          && (AVC_SLICE_TYPE2_I  != idr.type)
+          && (AVC_SLICE_TYPE2_SI != idr.type))
+      || (0 == idr.nal_ref_idc)
+      || (0 != sps.pic_order_cnt_type)) {
+    m_simple_picture_order = true;
+    // return;
+  }
+
   while (frames_end != frame_itr) {
     auto const &si = frame_itr->m_si;
 
@@ -1532,55 +1540,76 @@ mpeg4::p10::avc_es_parser_c::calculate_frame_order() {
   }
 }
 
+std::vector<int64_t>
+mpeg4::p10::avc_es_parser_c::calculate_provided_timestamps_to_use() {
+  auto frame_idx                     = 0u;
+  auto provided_timestamps_idx       = 0u;
+  auto const num_frames              = m_frames.size();
+  auto const num_provided_timestamps = m_provided_timestamps.size();
+
+  std::vector<int64_t> provided_timestamps_to_use;
+  provided_timestamps_to_use.reserve(num_frames);
+
+  while (   (frame_idx               < num_frames)
+         && (provided_timestamps_idx < num_provided_timestamps)) {
+    timestamp_c timestamp_to_use;
+    auto &frame = m_frames[frame_idx];
+
+    while (   (provided_timestamps_idx < num_provided_timestamps)
+           && (frame.m_position >= m_provided_timestamps[provided_timestamps_idx].second)) {
+      timestamp_to_use = timestamp_c::ns(m_provided_timestamps[provided_timestamps_idx].first);
+      ++provided_timestamps_idx;
+    }
+
+    if (timestamp_to_use.valid()) {
+      provided_timestamps_to_use.emplace_back(timestamp_to_use.to_ns());
+      frame.m_has_provided_timecode = true;
+    }
+
+    ++frame_idx;
+  }
+
+  mxdebug_if(m_debug_timecodes,
+             boost::format("cleanup; num frames %1% num provided timestamps available %2% num provided timestamps to use %3%\n"
+                           "  frames:\n%4%"
+                           "  provided timestamps (available):\n%5%"
+                           "  provided timestamps (to use):\n%6%")
+             % num_frames % num_provided_timestamps % provided_timestamps_to_use.size()
+             % boost::accumulate(m_frames, std::string{}, [](auto const &str, auto const &frame) {
+                 return str + (boost::format("    pos %1% size %2% type %3%\n") % frame.m_position % frame.m_data->get_size() % frame.m_type).str();
+               })
+             % boost::accumulate(m_provided_timestamps, std::string{}, [](auto const &str, auto const &provided_timestamp) {
+                 return str + (boost::format("    pos %1% timestamp %2%\n") % provided_timestamp.second % format_timestamp(provided_timestamp.first)).str();
+               })
+             % boost::accumulate(provided_timestamps_to_use, std::string{}, [](auto const &str, auto const &provided_timestamp) {
+                 return str + (boost::format("    timestamp %1%\n") % format_timestamp(provided_timestamp)).str();
+               }));
+
+  m_provided_timestamps.erase(m_provided_timestamps.begin(), m_provided_timestamps.begin() + provided_timestamps_idx);
+
+  brng::sort(provided_timestamps_to_use);
+
+  return provided_timestamps_to_use;
+}
+
 void
-mpeg4::p10::avc_es_parser_c::cleanup() {
-  auto s_debug_force_simple_picture_order = debugging_option_c{"avc_parser_force_simple_picture_order"};
-
-  if (m_frames.empty())
-    return;
-
-  if (m_discard_actual_frames) {
-    m_stats.num_frames_discarded    += m_frames.size();
-    m_stats.num_timecodes_discarded += m_provided_timecodes.size();
-
-    m_frames.clear();
-    m_provided_timecodes.clear();
-    m_provided_stream_positions.clear();
-
-    return;
-  }
-
-  auto const &idr        = m_frames.begin()->m_si;
-  auto const &sps        = m_sps_info_list[idr.sps];
-  m_simple_picture_order = false;
-
-  if (   (   (AVC_SLICE_TYPE_I   != idr.type)
-          && (AVC_SLICE_TYPE_SI  != idr.type)
-          && (AVC_SLICE_TYPE2_I  != idr.type)
-          && (AVC_SLICE_TYPE2_SI != idr.type))
-      || (0 == idr.nal_ref_idc)
-      || (0 != sps.pic_order_cnt_type)) {
-    m_simple_picture_order = true;
-    // return;
-  }
-
-  calculate_frame_order();
+mpeg4::p10::avc_es_parser_c::calculate_frame_timestamps() {
+  static auto s_debug_force_simple_picture_order = debugging_option_c{"avc_parser_force_simple_picture_order"};
+  auto provided_timestamps_to_use                = calculate_provided_timestamps_to_use();
 
   if (!m_simple_picture_order && !s_debug_force_simple_picture_order)
     brng::sort(m_frames, [](const avc_frame_t &f1, const avc_frame_t &f2) { return f1.m_presentation_order < f2.m_presentation_order; });
 
-  brng::sort(m_provided_timecodes);
-
-  auto frames_begin          = m_frames.begin();
-  auto frames_end            = m_frames.end();
-  auto frame_itr             = frames_begin;
-  auto previous_frame_itr    = frames_begin;
-  auto provided_timecode_itr = m_provided_timecodes.begin();
+  auto frames_begin            = m_frames.begin();
+  auto frames_end              = m_frames.end();
+  auto frame_itr               = frames_begin;
+  auto previous_frame_itr      = frames_begin;
+  auto provided_timestamps_itr = provided_timestamps_to_use.begin();
 
   while (frames_end != frame_itr) {
     if (frame_itr->m_has_provided_timecode) {
-      frame_itr->m_start = *provided_timecode_itr;
-      ++provided_timecode_itr;
+      frame_itr->m_start = *provided_timestamps_itr;
+      ++provided_timestamps_itr;
 
       if (frames_begin != frame_itr)
         previous_frame_itr->m_end = frame_itr->m_start;
@@ -1597,7 +1626,6 @@ mpeg4::p10::avc_es_parser_c::cleanup() {
   }
 
   m_max_timecode = m_frames.back().m_end;
-  m_provided_timecodes.erase(m_provided_timecodes.begin(), provided_timecode_itr);
 
   mxdebug_if(m_debug_timecodes, boost::format("CLEANUP frames <pres_ord dec_ord has_prov_tc tc dur>: %1%\n")
              % boost::accumulate(m_frames, std::string(""), [](std::string const &accu, avc_frame_t const &frame) {
@@ -1607,36 +1635,60 @@ mpeg4::p10::avc_es_parser_c::cleanup() {
 
   if (!m_simple_picture_order)
     brng::sort(m_frames, [](const avc_frame_t &f1, const avc_frame_t &f2) { return f1.m_decode_order < f2.m_decode_order; });
+}
 
-  frames_begin       = m_frames.begin();
-  frames_end         = m_frames.end();
+void
+mpeg4::p10::avc_es_parser_c::calculate_frame_references_and_update_stats() {
+  auto first_frame = true;
 
-  // This may be wrong but is needed for mkvmerge to work correctly
-  // (cluster_helper etc).
-  if (m_first_cleanup) {
-    frames_begin->m_keyframe = true;
-    m_first_cleanup          = false;
-  }
+  for (auto &frame : m_frames) {
+    if (!frame.is_i_frame() && !first_frame)
+      frame.m_ref1 = m_previous_frame_start_in_display_order - frame.m_start;
 
-  // mxinfo(boost::format("frame order calculation\n"));
+    m_previous_frame_start_in_display_order = frame.m_start;
+    m_duration_frequency[frame.m_end - frame.m_start]++;
 
-  for (frame_itr = frames_begin; frames_end != frame_itr; ++frame_itr) {
-    // mxinfo(boost::format("  type %4% decode order %1% presentation order %2% timestamp %3%\n") % frame_itr->m_decode_order % frame_itr->m_presentation_order % format_timestamp(frame_itr->m_start) % frame_itr->m_type);
-
-    if (!frame_itr->is_i_frame() && (frames_begin != frame_itr))
-      frame_itr->m_ref1 = m_previous_frame_start_in_display_order - frame_itr->m_start;
-
-    m_previous_frame_start_in_display_order = frame_itr->m_start;
-    m_duration_frequency[frame_itr->m_end - frame_itr->m_start]++;
-
-    if (frame_itr->m_si.field_pic_flag)
+    if (frame.m_si.field_pic_flag)
       ++m_stats.num_field_slices;
     else
       ++m_stats.num_field_slices;
+
+    first_frame = false;
+  }
+}
+
+void
+mpeg4::p10::avc_es_parser_c::cleanup() {
+  auto num_frames = m_frames.size();
+  if (!num_frames)
+    return;
+
+  if (m_discard_actual_frames) {
+    m_stats.num_frames_discarded    += m_frames.size();
+    m_stats.num_timecodes_discarded += m_provided_timestamps.size();
+
+    m_frames.clear();
+    m_provided_timestamps.clear();
+
+    return;
   }
 
+  calculate_frame_order();
+  calculate_frame_timestamps();
+  calculate_frame_references_and_update_stats();
+
+  if (m_first_cleanup && !m_frames.front().m_keyframe) {
+    // Drop all frames before the first key frames as they cannot be
+    // decoded anyway.
+    m_stats.num_frames_discarded += m_frames.size();
+    m_frames.clear();
+
+    return;
+  }
+
+  m_first_cleanup         = false;
   m_stats.num_frames_out += m_frames.size();
-  m_frames_out.insert(m_frames_out.end(), frames_begin, frames_end);
+  m_frames_out.insert(m_frames_out.end(), m_frames.begin(), m_frames.end());
   m_frames.clear();
 }
 
