@@ -57,10 +57,8 @@
 
 // This is ISO/IEC 13818-1.
 
-#define TS_CONSECUTIVE_PACKETS 16
-#define TS_PROBE_SIZE          (2 * TS_CONSECUTIVE_PACKETS * 204)
-#define TS_PACKET_SIZE         188
-#define TS_MAX_PACKET_SIZE     204
+#define TS_PACKET_SIZE     188
+#define TS_MAX_PACKET_SIZE 204
 
 namespace mtx { namespace mpeg_ts {
 
@@ -94,6 +92,7 @@ track_c::track_c(reader_c &p_reader,
   , a_sample_rate{}
   , a_bits_per_sample{}
   , a_bsid{}
+  , m_aac_multiplex_type{aac::parser_c::unknown_multiplex}
   , m_apply_dts_timestamp_fix{}
   , m_use_dts{}
   , m_timestamps_wrapped{}
@@ -332,16 +331,23 @@ int
 track_c::new_stream_a_aac() {
   add_pes_payload_to_probe_data();
 
+  auto pos = aac::parser_c::find_consecutive_frames(m_probe_data->get_buffer(), m_probe_data->get_size(), 5);
+  if (pos == -1)
+    return FILE_STATUS_MOREDATA;
+
   auto parser = aac::parser_c{};
-  parser.add_bytes(m_probe_data->get_buffer(), m_probe_data->get_size());
+  parser.add_bytes(m_probe_data->get_buffer() + pos, m_probe_data->get_size() - pos);
   if (!parser.frames_available() || !parser.headers_parsed())
     return FILE_STATUS_MOREDATA;
 
-  m_aac_frame   = parser.get_frame();
-  a_channels    = m_aac_frame.m_header.channels;
-  a_sample_rate = m_aac_frame.m_header.sample_rate;
+  m_aac_frame          = parser.get_frame();
+  m_aac_multiplex_type = parser.get_multiplex_type();
+  a_channels           = m_aac_frame.m_header.channels;
+  a_sample_rate        = m_aac_frame.m_header.sample_rate;
 
-  mxdebug_if(reader.m_debug_aac, boost::format("new_stream_a_aac: first AAC header: %1%\n") % m_aac_frame.to_string());
+  mxdebug_if(reader.m_debug_aac,
+             boost::format("new_stream_a_aac: found headers at %1% multiplex type %2% first header: %3%\n")
+             % pos % aac::parser_c::get_multiplex_type_name(m_aac_multiplex_type) % m_aac_frame.to_string());
 
   return 0;
 }
@@ -817,6 +823,93 @@ track_c::derive_hdmv_textst_pts_from_content() {
   return timestamp;
 }
 
+void
+track_c::determine_codec_from_stream_type(stream_type_e stream_type) {
+  switch (stream_type) {
+    case stream_type_e::iso_11172_video:
+    case stream_type_e::iso_13818_video:
+      type  = pid_type_e::video;
+      codec = codec_c::look_up(codec_c::type_e::V_MPEG12);
+      break;
+    case stream_type_e::iso_14496_part2_video:
+      type  = pid_type_e::video;
+      codec = codec_c::look_up(codec_c::type_e::V_MPEG4_P2);
+      break;
+    case stream_type_e::iso_14496_part10_video:
+      type  = pid_type_e::video;
+      codec = codec_c::look_up(codec_c::type_e::V_MPEG4_P10);
+      break;
+    case stream_type_e::iso_23008_part2_video:
+      type  = pid_type_e::video;
+      codec = codec_c::look_up(codec_c::type_e::V_MPEGH_P2);
+      break;
+    case stream_type_e::stream_video_vc1:
+      type  = pid_type_e::video;
+      codec = codec_c::look_up(codec_c::type_e::V_VC1);
+      break;
+    case stream_type_e::iso_11172_audio:
+    case stream_type_e::iso_13818_audio:
+      type  = pid_type_e::audio;
+      codec = codec_c::look_up(codec_c::type_e::A_MP3);
+      break;
+    case stream_type_e::iso_13818_part7_audio:
+    case stream_type_e::iso_14496_part3_audio:
+      type  = pid_type_e::audio;
+      codec = codec_c::look_up(codec_c::type_e::A_AAC);
+      break;
+    case stream_type_e::stream_audio_pcm:
+      type  = pid_type_e::audio;
+      codec = codec_c::look_up(codec_c::type_e::A_PCM);
+      break;
+
+    case stream_type_e::stream_audio_ac3_lossless: {
+      auto ac3_track       = std::make_shared<track_c>(*this);
+      ac3_track->type      = pid_type_e::audio;
+      ac3_track->codec     = codec_c::look_up(codec_c::type_e::A_AC3);
+      ac3_track->converter = std::make_shared<truehd_ac3_splitting_packet_converter_c>();
+      ac3_track->m_master  = this;
+      ac3_track->set_pid(pid);
+
+      type                 = pid_type_e::audio;
+      codec                = codec_c::look_up(codec_c::type_e::A_TRUEHD);
+      converter            = ac3_track->converter;
+      m_coupled_tracks.push_back(ac3_track);
+
+      break;
+    }
+
+    case stream_type_e::stream_audio_ac3:
+    case stream_type_e::stream_audio_eac3:      // E-AC-3
+    case stream_type_e::stream_audio_eac3_2:    // E-AC-3 secondary stream
+    case stream_type_e::stream_audio_eac3_atsc: // E-AC-3 as defined in ATSC A/52:2012 Annex G
+      type      = pid_type_e::audio;
+      codec     = codec_c::look_up(codec_c::type_e::A_AC3);
+      break;
+    case stream_type_e::stream_audio_dts:
+    case stream_type_e::stream_audio_dts_hd:
+    case stream_type_e::stream_audio_dts_hd_ma:
+    case stream_type_e::stream_audio_dts_hd2:
+      type      = pid_type_e::audio;
+      codec     = codec_c::look_up(codec_c::type_e::A_DTS);
+      break;
+    case stream_type_e::stream_subtitles_hdmv_pgs:
+      type      = pid_type_e::subtitles;
+      codec     = codec_c::look_up(codec_c::type_e::S_HDMV_PGS);
+      probed_ok = true;
+      break;
+    case stream_type_e::stream_subtitles_hdmv_textst:
+      type      = pid_type_e::subtitles;
+      codec     = codec_c::look_up(codec_c::type_e::S_HDMV_TEXTST);
+      break;
+    case stream_type_e::iso_13818_pes_private:
+      break;
+    default:
+      mxdebug_if(reader.m_debug_pat_pmt, boost::format("parse_pmt: Unknown stream type: %1%\n") % static_cast<int>(stream_type));
+      type      = pid_type_e::unknown;
+      break;
+  }
+}
+
 void
 track_c::reset_processing_state() {
   m_timestamp.reset();
@@ -882,27 +975,21 @@ reader_c::probe_file(mm_io_c *in,
 
   auto packet_size = detect_packet_size(in_to_use, in_to_use->get_size());
 
-  if (packet_size == -1)
-    return false;
-
-  // Check for a h.264/h.265 start code right at the beginning in
-  // order to avoid false positives.
-  in_to_use->setFilePointer(0);
-  auto marker = in_to_use->read_uint32_be();
-
-  if ((marker == 0x00000001u) || ((marker >> 8) == 0x00000001u))
-    return false;
-
-  return true;
+  return packet_size > 0;
 }
 
 int
 reader_c::detect_packet_size(mm_io_c *in,
                              uint64_t size) {
+  debugging_option_c debug{"mpeg_ts|mpeg_ts_packet_size"};
+
   try {
-    size        = std::min(static_cast<uint64_t>(TS_PROBE_SIZE), size);
-    auto buffer = memory_c::alloc(size);
-    auto mem    = buffer->get_buffer();
+    size                         = std::min<uint64_t>(1024 * 1024, size);
+    auto num_startcodes_required = std::max<uint64_t>(size / 3 / TS_MAX_PACKET_SIZE, 32);
+    auto buffer                  = memory_c::alloc(size);
+    auto mem                     = buffer->get_buffer();
+
+    mxdebug_if(debug, boost::format("detect_packet_size: size to probe %1% num required startcodes %2%\n") % size % num_startcodes_required);
 
     in->setFilePointer(0, seek_beginning);
     size = in->read(mem, size);
@@ -918,18 +1005,22 @@ reader_c::detect_packet_size(mm_io_c *in,
         unsigned int packet_size    = potential_packet_sizes[k];
         unsigned int num_startcodes = 1;
 
-        while ((TS_CONSECUTIVE_PACKETS > num_startcodes) && (pos < size) && (0x47 == mem[pos])) {
+        while ((num_startcodes_required > num_startcodes) && (pos < size) && (0x47 == mem[pos])) {
           pos += packet_size;
           ++num_startcodes;
         }
 
-        if (TS_CONSECUTIVE_PACKETS <= num_startcodes)
+        if (num_startcodes_required <= num_startcodes) {
+          mxdebug_if(debug, boost::format("detect_packet_size: detected packet size %1% at offset %2%\n") % packet_size % positions[i]);
           return packet_size;
+        }
       }
     }
   } catch (...) {
   }
 
+  mxdebug_if(debug, "detect_packet_size: packet size could not be determined\n");
+
   return -1;
 }
 
@@ -967,8 +1058,10 @@ reader_c::read_headers_for_file(std::size_t file_num) {
   try {
     auto file_size           = f.m_in->get_size();
     f.m_probe_range          = calculate_probe_range(file_size, 10 * 1024 * 1024);
-    size_t size_to_probe     = std::min<int64_t>(file_size, f.m_probe_range);
+    auto size_to_probe       = std::min<uint64_t>(file_size,     f.m_probe_range);
+    auto min_size_to_probe   = std::min<uint64_t>(size_to_probe, 5 * 1024 * 1024);
     f.m_detected_packet_size = detect_packet_size(f.m_in.get(), size_to_probe);
+    f.m_ignored_pids[0]      = true; // PAT
 
     f.m_in->setFilePointer(0);
 
@@ -990,7 +1083,10 @@ reader_c::read_headers_for_file(std::size_t file_num) {
 
       parse_packet(buf);
 
-      if (f.m_pat_found && f.m_pmt_found && (0 == f.m_es_to_process))
+      if (   f.m_pat_found
+          && f.m_pmt_found
+          && (0 == f.m_es_to_process)
+          && (f.m_in->getFilePointer() >= min_size_to_probe))
         break;
 
       auto eof = f.m_in->eof() || (f.m_in->getFilePointer() >= size_to_probe);
@@ -1028,6 +1124,14 @@ reader_c::read_headers_for_file(std::size_t file_num) {
 
   f.m_in->setFilePointer(0, seek_beginning); // rewind file for later remux
 
+  // Run probe_packet_complete() for track-type detection once for
+  // each track. This way tracks that don't actually need their
+  // content to be found during probing will be set to OK, too.
+  for (auto const &track : m_tracks) {
+    track->clear_pes_payload();
+    probe_packet_complete(*track);
+  }
+
   brng::copy(m_tracks, std::back_inserter(m_all_probed_tracks));
 }
 
@@ -1289,8 +1393,9 @@ reader_c::parse_pat(track_c &track) {
       if (skip == true)
         continue;
 
-      auto pmt          = std::make_shared<track_c>(*this, pid_type_e::pmt);
-      f.m_es_to_process = 0;
+      auto pmt                  = std::make_shared<track_c>(*this, pid_type_e::pmt);
+      f.m_es_to_process         = 0;
+      f.m_ignored_pids[tmp_pid] = true;
 
       pmt->set_pid(tmp_pid);
 
@@ -1301,15 +1406,108 @@ reader_c::parse_pat(track_c &track) {
   return true;
 }
 
+memory_cptr
+reader_c::read_pmt_descriptor(mm_io_c &io) {
+  auto buffer = io.read(sizeof(pmt_descriptor_t));
+  auto length = buffer->get_buffer()[1];
+
+  buffer->resize(sizeof(pmt_descriptor_t) + length);
+  if (io.read(buffer->get_buffer() + sizeof(pmt_descriptor_t), length) != length)
+    throw mtx::mm_io::end_of_file_x{};
+
+  return buffer;
+}
+
+bool
+reader_c::parse_pmt_pid_info(mm_mem_io_c &mem) {
+  auto &f                  = file();
+  auto missing_tag         = true;
+
+  auto pmt_pid_info_buffer = mem.read(sizeof(pmt_pid_info_t));
+  auto pmt_pid_info        = reinterpret_cast<pmt_pid_info_t *>(pmt_pid_info_buffer->get_buffer());
+  auto es_info_length      = pmt_pid_info->get_es_info_length();
+
+  auto track               = std::make_shared<track_c>(*this);
+  track->type              = pid_type_e::unknown;
+
+  track->set_pid(pmt_pid_info->get_pid());
+  track->determine_codec_from_stream_type(pmt_pid_info->stream_type);
+
+  while (es_info_length >= sizeof(pmt_descriptor_t)) {
+    auto pmt_descriptor_buffer = read_pmt_descriptor(mem);
+    auto pmt_descriptor        = reinterpret_cast<pmt_descriptor_t *>(pmt_descriptor_buffer->get_buffer());
+
+    mxdebug_if(m_debug_pat_pmt, boost::format("parse_pmt: PMT descriptor tag 0x%|1$02x| length %2%\n") % static_cast<unsigned int>(pmt_descriptor->tag) % static_cast<unsigned int>(pmt_descriptor->length));
+
+    if (pmt_descriptor_buffer->get_size() > es_info_length)
+      break;
+
+    es_info_length -= pmt_descriptor_buffer->get_size();
+
+    if (0x0a != pmt_descriptor->tag)
+      missing_tag = false;
+
+    switch (pmt_descriptor->tag) {
+      case 0x05: // registration descriptor
+        track->parse_registration_pmt_descriptor(*pmt_descriptor, *pmt_pid_info);
+        break;
+      case 0x0a: // ISO 639 language descriptor
+        track->parse_iso639_language_from(pmt_descriptor + 1);
+        break;
+      case 0x56: // Teletext descriptor
+        track->parse_srt_pmt_descriptor(*pmt_descriptor, *pmt_pid_info);
+        break;
+      case 0x59: // Subtitles descriptor
+        track->parse_subtitling_pmt_descriptor(*pmt_descriptor, *pmt_pid_info);
+        break;
+      case 0x6A: // AC-3 descriptor
+      case 0x7A: // E-AC-3 descriptor
+        track->parse_ac3_pmt_descriptor(*pmt_descriptor, *pmt_pid_info);
+        break;
+      case 0x7b: // DTS descriptor
+        track->parse_dts_pmt_descriptor(*pmt_descriptor, *pmt_pid_info);
+        break;
+    }
+  }
+
+  // Default to AC-3 if it's a PES private stream type that's missing
+  // a known/more concrete descriptor tag.
+  if ((pmt_pid_info->stream_type == stream_type_e::iso_13818_pes_private) && missing_tag) {
+    track->type  = pid_type_e::audio;
+    track->codec = codec_c::look_up(codec_c::type_e::A_AC3);
+  }
+
+  if (track->type != pid_type_e::unknown) {
+    f.m_pmt_found    = true;
+    track->processed = false;
+    m_tracks.push_back(track);
+    ++f.m_es_to_process;
+
+    brng::copy(track->m_coupled_tracks, std::back_inserter(m_tracks));
+    f.m_es_to_process += track->m_coupled_tracks.size();
+  }
+
+  mxdebug_if(m_debug_pat_pmt,
+             boost::format("parse_pmt: PID %1% stream type %|2$02x| has type: %3%\n")
+             % track->pid
+             % static_cast<unsigned int>(pmt_pid_info->stream_type)
+             % (track->type != pid_type_e::unknown ? track->codec.get_name() : "<unknown>"));
+
+  return true;
+}
+
 bool
 reader_c::parse_pmt(track_c &track) {
-  if (track.pes_payload_read->get_size() < sizeof(pmt_t)) {
-    mxdebug_if(m_debug_pat_pmt, "Invalid parameters!\n");
+  auto payload_size = track.pes_payload_read->get_size();
+  auto pmt_start    = track.pes_payload_read->get_buffer();
+  auto pmt_end      = pmt_start + payload_size;
+
+  if (payload_size < sizeof(pmt_t)) {
+    mxdebug_if(m_debug_pat_pmt, boost::format("Actual PMT size %1% less than PMT structure size %2%!\n") % payload_size % sizeof(pmt_t));
     return false;
   }
 
-  auto pmt        = track.pes_payload_read->get_buffer();
-  auto pmt_header = reinterpret_cast<pmt_t *>(pmt);
+  auto pmt_header = reinterpret_cast<pmt_t *>(pmt_start);
 
   if (pmt_header->table_id != 0x02) {
     mxdebug_if(m_debug_pat_pmt, "Invalid PMT table_id!\n");
@@ -1326,10 +1524,31 @@ reader_c::parse_pmt(track_c &track) {
     return false;
   }
 
-  auto &f                           = file();
-  unsigned short pmt_section_length = pmt_header->get_section_length();
-  uint32_t elapsed_CRC              = calculate_crc(pmt, 3 + pmt_section_length - 4/*CRC32*/);
-  uint32_t read_CRC                 = get_uint32_be(pmt + 3 + pmt_section_length - 4);
+  auto &f                  = file();
+  auto pmt_section_length  = pmt_header->get_section_length();
+  auto pmt_section_start   = pmt_start         + offsetof(pmt_t, program_number);
+  auto pmt_section_end     = pmt_section_start + pmt_section_length;
+  auto program_info_length = pmt_header->get_program_info_length();
+  auto program_info_start  = pmt_start + sizeof(pmt_t);
+  auto program_info_end    = program_info_start + program_info_length;
+
+  if (program_info_end > pmt_end) {
+    mxdebug_if(m_debug_pat_pmt, boost::format("Actual PMT size %1% less than PMT program info size + program info offset %2%!\n") % payload_size % (pmt_end - pmt_start));
+    return false;
+  }
+
+  if (pmt_section_end > pmt_end) {
+    mxdebug_if(m_debug_pat_pmt, boost::format("Actual PMT size %1% less than PMT section size + section offset %2%!\n") % payload_size % (pmt_end - pmt_start));
+    return false;
+  }
+
+  if ((pmt_section_length < 13) || (pmt_section_length > 1021)) {
+    mxdebug_if(m_debug_pat_pmt, boost::format("parse_pmt: Wrong PMT section_length %1%\n") % pmt_section_length);
+    return false;
+  }
+
+  uint32_t elapsed_CRC = calculate_crc(pmt_start, 3 + pmt_section_length - 4/*CRC32*/);
+  uint32_t read_CRC    = get_uint32_be(pmt_section_end - 4);
 
   if (elapsed_CRC != read_CRC) {
     mxdebug_if(m_debug_pat_pmt, boost::format("parse_pmt: Wrong PMT CRC !!! Elapsed = 0x%|1$08x|, read 0x%|2$08x|, validate PMT CRC? %3%\n") % elapsed_CRC % read_CRC % f.m_validate_pmt_crc);
@@ -1338,187 +1557,21 @@ reader_c::parse_pmt(track_c &track) {
       return false;
   }
 
-  if (pmt_section_length < 13 || pmt_section_length > 1021) {
-    mxdebug_if(m_debug_pat_pmt, boost::format("parse_pmt: Wrong PMT section_length (=%1%)\n") % pmt_section_length);
-    return false;
+  mxdebug_if(m_debug_pat_pmt,
+             boost::format("parse_pmt: program number: %1% PCR PID: %2% program info length: %3%\n")
+             % pmt_header->get_program_number() % pmt_header->get_pcr_pid() % program_info_length);
+
+  mm_mem_io_c mem{program_info_end, static_cast<uint64_t>(pmt_section_end - 4 - program_info_end)};
+
+  try {
+    bool ok = true;
+    while (ok)
+      ok = parse_pmt_pid_info(mem);
+
+  } catch (mtx::mm_io::exception &) {
   }
 
-  auto pmt_descriptor      = reinterpret_cast<pmt_descriptor_t *>(pmt + sizeof(pmt_t));
-  auto program_info_length = pmt_header->get_program_info_length();
-
-  while (pmt_descriptor < (pmt_descriptor_t *)(pmt + sizeof(pmt_t) + program_info_length))
-    pmt_descriptor = (pmt_descriptor_t *)((unsigned char *)pmt_descriptor + sizeof(pmt_descriptor_t) + pmt_descriptor->length);
-
-  pmt_pid_info_t *pmt_pid_info = (pmt_pid_info_t *)pmt_descriptor;
-
-  // Calculate pids_count
-  size_t pids_found = 0;
-  while (pmt_pid_info < (pmt_pid_info_t *)(pmt + 3 + pmt_section_length - 4/*CRC32*/)) {
-    pids_found++;
-    pmt_pid_info = (pmt_pid_info_t *)((unsigned char *)pmt_pid_info + sizeof(pmt_pid_info_t) + pmt_pid_info->get_es_info_length());
-  }
-
-  mxdebug_if(m_debug_pat_pmt, boost::format("parse_pmt: program number     (%1%)\n") % pmt_header->get_program_number());
-  mxdebug_if(m_debug_pat_pmt, boost::format("parse_pmt: pcr pid            (%1%)\n") % pmt_header->get_pcr_pid());
-
-  if (pids_found == 0) {
-    mxdebug_if(m_debug_pat_pmt, "There's no information about elementary PIDs\n");
-    return 0;
-  }
-
-  pmt_pid_info = (pmt_pid_info_t *)pmt_descriptor;
-
-  // Extract pid_info
-  while (pmt_pid_info < (pmt_pid_info_t *)(pmt + 3 + pmt_section_length - 4/*CRC32*/)) {
-    track_ptr track(new track_c(*this));
-    unsigned short es_info_length = pmt_pid_info->get_es_info_length();
-    track->type                   = pid_type_e::unknown;
-
-    track->set_pid(pmt_pid_info->get_pid());
-
-    switch(pmt_pid_info->stream_type) {
-      case stream_type_e::iso_11172_video:
-      case stream_type_e::iso_13818_video:
-        track->type      = pid_type_e::video;
-        track->codec     = codec_c::look_up(codec_c::type_e::V_MPEG12);
-        break;
-      case stream_type_e::iso_14496_part2_video:
-        track->type      = pid_type_e::video;
-        track->codec     = codec_c::look_up(codec_c::type_e::V_MPEG4_P2);
-        break;
-      case stream_type_e::iso_14496_part10_video:
-        track->type      = pid_type_e::video;
-        track->codec     = codec_c::look_up(codec_c::type_e::V_MPEG4_P10);
-        break;
-      case stream_type_e::iso_23008_part2_video:
-        track->type      = pid_type_e::video;
-        track->codec     = codec_c::look_up(codec_c::type_e::V_MPEGH_P2);
-        break;
-      case stream_type_e::stream_video_vc1:
-        track->type      = pid_type_e::video;
-        track->codec     = codec_c::look_up(codec_c::type_e::V_VC1);
-        break;
-      case stream_type_e::iso_11172_audio:
-      case stream_type_e::iso_13818_audio:
-        track->type      = pid_type_e::audio;
-        track->codec     = codec_c::look_up(codec_c::type_e::A_MP3);
-        break;
-      case stream_type_e::iso_13818_part7_audio:
-      case stream_type_e::iso_14496_part3_audio:
-        track->type      = pid_type_e::audio;
-        track->codec     = codec_c::look_up(codec_c::type_e::A_AAC);
-        break;
-      case stream_type_e::stream_audio_pcm:
-        track->type      = pid_type_e::audio;
-        track->codec     = codec_c::look_up(codec_c::type_e::A_PCM);
-        break;
-
-      case stream_type_e::stream_audio_ac3_lossless: {
-        auto ac3_track         = std::make_shared<track_c>(*this);
-        ac3_track->type        = pid_type_e::audio;
-        ac3_track->codec       = codec_c::look_up(codec_c::type_e::A_AC3);
-        ac3_track->converter   = std::make_shared<truehd_ac3_splitting_packet_converter_c>();
-        ac3_track->m_master    = track.get();
-        ac3_track->set_pid(pmt_pid_info->get_pid());
-
-        track->type            = pid_type_e::audio;
-        track->codec           = codec_c::look_up(codec_c::type_e::A_TRUEHD);
-        track->converter       = ac3_track->converter;
-        track->m_coupled_tracks.push_back(ac3_track);
-
-        break;
-      }
-
-      case stream_type_e::stream_audio_ac3:
-      case stream_type_e::stream_audio_eac3:      // E-AC-3
-      case stream_type_e::stream_audio_eac3_2:    // E-AC-3 secondary stream
-      case stream_type_e::stream_audio_eac3_atsc: // E-AC-3 as defined in ATSC A/52:2012 Annex G
-        track->type      = pid_type_e::audio;
-        track->codec     = codec_c::look_up(codec_c::type_e::A_AC3);
-        break;
-      case stream_type_e::stream_audio_dts:
-      case stream_type_e::stream_audio_dts_hd:
-      case stream_type_e::stream_audio_dts_hd_ma:
-      case stream_type_e::stream_audio_dts_hd2:
-        track->type      = pid_type_e::audio;
-        track->codec     = codec_c::look_up(codec_c::type_e::A_DTS);
-        break;
-      case stream_type_e::stream_subtitles_hdmv_pgs:
-        track->type      = pid_type_e::subtitles;
-        track->codec     = codec_c::look_up(codec_c::type_e::S_HDMV_PGS);
-        track->probed_ok = true;
-        break;
-      case stream_type_e::stream_subtitles_hdmv_textst:
-        track->type      = pid_type_e::subtitles;
-        track->codec     = codec_c::look_up(codec_c::type_e::S_HDMV_TEXTST);
-        break;
-      case stream_type_e::iso_13818_pes_private:
-        break;
-      default:
-        mxdebug_if(m_debug_pat_pmt, boost::format("parse_pmt: Unknown stream type: %1%\n") % (int)pmt_pid_info->stream_type);
-        track->type      = pid_type_e::unknown;
-        break;
-    }
-
-    pmt_descriptor   = (pmt_descriptor_t *)((unsigned char *)pmt_pid_info + sizeof(pmt_pid_info_t));
-    bool missing_tag = true;
-
-    while (pmt_descriptor < (pmt_descriptor_t *)((unsigned char *)pmt_pid_info + sizeof(pmt_pid_info_t) + es_info_length)) {
-      mxdebug_if(m_debug_pat_pmt, boost::format("parse_pmt: PMT descriptor tag 0x%|1$02x| length %2%\n") % static_cast<unsigned int>(pmt_descriptor->tag) % static_cast<unsigned int>(pmt_descriptor->length));
-
-      if (0x0a != pmt_descriptor->tag)
-        missing_tag = false;
-
-      switch (pmt_descriptor->tag) {
-        case 0x05: // registration descriptor
-          track->parse_registration_pmt_descriptor(*pmt_descriptor, *pmt_pid_info);
-          break;
-        case 0x0a: // ISO 639 language descriptor
-          track->parse_iso639_language_from(pmt_descriptor + 1);
-          break;
-        case 0x56: // Teletext descriptor
-          track->parse_srt_pmt_descriptor(*pmt_descriptor, *pmt_pid_info);
-          break;
-        case 0x59: // Subtitles descriptor
-          track->parse_subtitling_pmt_descriptor(*pmt_descriptor, *pmt_pid_info);
-          break;
-        case 0x6A: // AC-3 descriptor
-        case 0x7A: // E-AC-3 descriptor
-          track->parse_ac3_pmt_descriptor(*pmt_descriptor, *pmt_pid_info);
-          break;
-        case 0x7b: // DTS descriptor
-          track->parse_dts_pmt_descriptor(*pmt_descriptor, *pmt_pid_info);
-          break;
-      }
-
-      pmt_descriptor = (pmt_descriptor_t *)((unsigned char *)pmt_descriptor + sizeof(pmt_descriptor_t) + pmt_descriptor->length);
-    }
-
-    // Default to AC-3 if it's a PES private stream type that's missing
-    // a known/more concrete descriptor tag.
-    if ((pmt_pid_info->stream_type == stream_type_e::iso_13818_pes_private) && missing_tag) {
-      track->type  = pid_type_e::audio;
-      track->codec = codec_c::look_up(codec_c::type_e::A_AC3);
-    }
-
-    if (track->type != pid_type_e::unknown) {
-      f.m_pmt_found    = true;
-      track->processed = false;
-      m_tracks.push_back(track);
-      ++f.m_es_to_process;
-
-      brng::copy(track->m_coupled_tracks, std::back_inserter(m_tracks));
-      f.m_es_to_process += track->m_coupled_tracks.size();
-    }
-
-    mxdebug_if(m_debug_pat_pmt,
-               boost::format("parse_pmt: PID %1% stream type %|2$02x| has type: %3%\n")
-               % track->pid
-               % static_cast<unsigned int>(pmt_pid_info->stream_type)
-               % (track->type != pid_type_e::unknown ? track->codec.get_name() : "<unknown>"));
-
-    pmt_pid_info = (pmt_pid_info_t *)((unsigned char *)pmt_pid_info + sizeof(pmt_pid_info_t) + es_info_length);
-  }
+  f.m_ignored_pids[track.pid] = true;
 
   return true;
 }
@@ -1590,7 +1643,7 @@ reader_c::parse_pes(track_c &track) {
 
   if (processing_state_e::determining_timestamp_offset == f.m_state) {
     if (   dts.valid()
-        && (track.type != pid_type_e::subtitles)
+        && (mtx::included_in(track.type, pid_type_e::audio, pid_type_e::video))
         && (   !f.m_global_timestamp_offset.valid()
             || (dts < f.m_global_timestamp_offset)))
       f.m_global_timestamp_offset = dts;
@@ -1611,7 +1664,7 @@ reader_c::parse_pes(track_c &track) {
   if (pts.valid()) {
     track.m_previous_valid_timestamp = pts;
 
-    if (   (track.type != pid_type_e::subtitles)
+    if (   mtx::included_in(track.type, pid_type_e::audio, pid_type_e::video)
         && (   !f.m_global_timestamp_offset.valid()
             || (dts < f.m_global_timestamp_offset))) {
       mxdebug_if(m_debug_headers, boost::format("new global timestamp offset %1% prior %2% file position afterwards %3%\n") % dts % f.m_global_timestamp_offset % f.m_in->getFilePointer());
@@ -1639,11 +1692,35 @@ reader_c::read_timestamp(unsigned char *p) {
   return timestamp_c::mpeg(mpeg_timestamp);
 }
 
+track_ptr
+reader_c::handle_packet_for_pid_not_listed_in_pmt(uint16_t pid) {
+  auto &f = file();
+
+  if (   (f.m_state != processing_state_e::probing)
+      || f.m_ignored_pids[pid]
+      || !f.m_pat_found
+      || !f.m_pmt_found)
+    return {};
+
+  mxdebug_if(m_debug_pat_pmt, boost::format("found packet for track PID %1% not listed in PMT, attempting type detection by content\n") % pid);
+
+  auto track = std::make_shared<track_c>(*this);
+  track->set_pid(pid);
+
+  m_tracks.push_back(track);
+  ++f.m_es_to_process;
+
+  return track;
+}
+
 void
 reader_c::parse_packet(unsigned char *buf) {
   auto hdr   = reinterpret_cast<packet_header_t *>(buf);
   auto track = find_track_for_pid(hdr->get_pid());
 
+  if (!track)
+    track = handle_packet_for_pid_not_listed_in_pmt(hdr->get_pid());
+
   if (   hdr->has_transport_error() // corrupted packet
       || !hdr->has_payload()        // no ts_payload
       || !track
@@ -1664,7 +1741,7 @@ reader_c::handle_ts_payload(track_c &track,
   if (mtx::included_in(track.type, pid_type_e::pat, pid_type_e::pmt))
     handle_pat_pmt_payload(track, ts_header, ts_payload, ts_payload_size);
 
-  else if (mtx::included_in(track.type, pid_type_e::video, pid_type_e::audio, pid_type_e::subtitles))
+  else if (mtx::included_in(track.type, pid_type_e::video, pid_type_e::audio, pid_type_e::subtitles, pid_type_e::unknown))
     handle_pes_payload(track, ts_header, ts_payload, ts_payload_size);
 
   else {
@@ -1733,6 +1810,31 @@ reader_c::handle_pes_payload(track_c &track,
     parse_pes(track);
 }
 
+void
+reader_c::determine_track_type_by_pes_content(track_c &track) {
+  auto buffer = track.pes_payload_read->get_buffer();
+  auto size   = track.pes_payload_read->get_size();
+
+  if (!size)
+    return;
+
+  if (m_debug_pat_pmt) {
+    auto data = to_hex(buffer, std::min<unsigned int>(size, 4));
+    mxdebug(boost::format("PID %1%: attempting type detection from content for with size %2%; first four bytes: %3%\n") % track.pid % size % data);
+  }
+
+  if ((size >= 3) && ((get_uint24_be(buffer) & AAC_ADTS_SYNC_WORD_MASK) == AAC_ADTS_SYNC_WORD)) {
+    track.type  = pid_type_e::audio;
+    track.codec = codec_c::look_up(codec_c::type_e::A_AAC);
+
+  } else if ((size >= 2) && (get_uint16_be(buffer) == AC3_SYNC_WORD)) {
+    track.type  = pid_type_e::audio;
+    track.codec = codec_c::look_up(codec_c::type_e::A_AC3);
+  }
+
+  mxdebug_if(m_debug_pat_pmt, boost::format("Detected type: %1% codec: %2%\n") % static_cast<unsigned int>(track.type) % track.codec);
+}
+
 int
 reader_c::determine_track_parameters(track_c &track) {
   if (track.type == pid_type_e::pat)
@@ -1741,7 +1843,10 @@ reader_c::determine_track_parameters(track_c &track) {
   else if (track.type == pid_type_e::pmt)
     return parse_pmt(track) ? 0 : -1;
 
-  else if (track.type == pid_type_e::video) {
+  else if (track.type == pid_type_e::unknown)
+    determine_track_type_by_pes_content(track);
+
+  if (track.type == pid_type_e::video) {
     if (track.codec.is(codec_c::type_e::V_MPEG12))
       return track.new_stream_v_mpeg_1_2();
     else if (track.codec.is(codec_c::type_e::V_MPEG4_P10))
@@ -1781,6 +1886,11 @@ reader_c::determine_track_parameters(track_c &track) {
 
 void
 reader_c::probe_packet_complete(track_c &track) {
+  if (track.probed_ok) {
+    track.clear_pes_payload();
+    return;
+  }
+
   int result = -1;
 
   try {
@@ -1889,7 +1999,7 @@ void
 reader_c::create_aac_audio_packetizer(track_ptr const &track) {
   auto aac_packetizer = new aac_packetizer_c(this, m_ti, track->m_aac_frame.m_header.profile, track->m_aac_frame.m_header.sample_rate, track->m_aac_frame.m_header.channels, true);
   track->ptzr         = add_packetizer(aac_packetizer);
-  track->converter.reset(new aac_framing_packet_converter_c{PTZR(track->ptzr)});
+  track->converter    = std::make_shared<aac_framing_packet_converter_c>(PTZR(track->ptzr), track->m_aac_multiplex_type);
 
   if (AAC_PROFILE_SBR == track->m_aac_frame.m_header.profile)
     aac_packetizer->set_audio_output_sampling_freq(track->m_aac_frame.m_header.sample_rate * 2);
