@@ -301,6 +301,7 @@ public:
   int a_channels, a_sample_rate, a_bits_per_sample, a_bsid;
   mtx::dts::header_t a_dts_header;
   aac::frame_c m_aac_frame;
+  aac::parser_c::multiplex_type_e m_aac_multiplex_type;
 
   bool m_apply_dts_timestamp_fix, m_use_dts, m_timestamps_wrapped, m_truehd_found_truehd, m_truehd_found_ac3;
   std::vector<track_ptr> m_coupled_tracks;
@@ -365,6 +366,8 @@ public:
   timestamp_c derive_pts_from_content();
   timestamp_c derive_hdmv_textst_pts_from_content();
 
+  void determine_codec_from_stream_type(stream_type_e stream_type);
+
   void process(packet_cptr const &packet);
 
   void parse_iso639_language_from(void const *buffer);
@@ -376,6 +379,7 @@ struct file_t {
   mm_io_cptr m_in;
 
   std::unordered_map<uint16_t, track_ptr> m_pid_to_track_map;
+  std::unordered_map<uint16_t, bool> m_ignored_pids;
   std::vector<generic_packetizer_c *> m_packetizers;
 
   bool m_pat_found, m_pmt_found;
@@ -443,12 +447,15 @@ private:
   void handle_ts_payload(track_c &track, packet_header_t &ts_header, unsigned char *ts_payload, std::size_t ts_payload_size);
   void handle_pat_pmt_payload(track_c &track, packet_header_t &ts_header, unsigned char *ts_payload, std::size_t ts_payload_size);
   void handle_pes_payload(track_c &track, packet_header_t &ts_header, unsigned char *ts_payload, std::size_t ts_payload_size);
+  track_ptr handle_packet_for_pid_not_listed_in_pmt(uint16_t pid);
 
   bool parse_pat(track_c &track);
   bool parse_pmt(track_c &track);
+  bool parse_pmt_pid_info(mm_mem_io_c &mem);
   void parse_pes(track_c &track);
   void probe_packet_complete(track_c &track);
   int determine_track_parameters(track_c &track);
+  void determine_track_type_by_pes_content(track_c &track);
 
   file_status_e finish();
   bool all_files_done() const;
@@ -484,6 +491,8 @@ private:
 
   void add_multiplexed_ids(std::vector<uint64_t> &multiplexed_ids, track_c &track);
 
+  static memory_cptr read_pmt_descriptor(mm_io_c &io);
+
   friend class track_c;
 };
 
