@@ -100,7 +100,6 @@ struct qt_chunkmap_t {
 struct qt_editlist_t {
   int64_t  segment_duration{}, media_time{};
   uint16_t media_rate_integer{}, media_rate_fraction{};
-  int64_t  frames{}, start_sample{}, start_frame{}, pts_offset{};
 };
 
 struct qt_sample_t {
@@ -124,8 +123,8 @@ struct qt_sample_t {
 };
 
 struct qt_frame_offset_t {
-  uint32_t count;
-  uint32_t offset;
+  unsigned int count;
+  int64_t offset;
 
   qt_frame_offset_t()
     : count{}
@@ -133,7 +132,7 @@ struct qt_frame_offset_t {
   {
   }
 
-  qt_frame_offset_t(uint32_t p_count, uint32_t p_offset)
+  qt_frame_offset_t(unsigned int p_count, int64_t p_offset)
     : count{p_count}
     , offset{p_offset}
   {
@@ -228,7 +227,7 @@ struct qtmp4_demuxer_c {
   codec_c codec;
   pcm_packetizer_c::pcm_format_e m_pcm_format;
 
-  int64_t time_scale, duration, global_duration, constant_editlist_offset_ns, num_frames_from_trun;
+  int64_t time_scale, duration, global_duration, num_frames_from_trun;
   uint32_t sample_size;
 
   std::vector<qt_sample_t> sample_table;
@@ -256,6 +255,7 @@ struct qtmp4_demuxer_c {
   memory_cptr stsd;
   unsigned int stsd_non_priv_struct_size;
   uint32_t v_width, v_height, v_bitdepth, v_display_width_flt{}, v_display_height_flt{};
+  uint16_t v_colour_primaries, v_colour_transfer_characteristics, v_colour_matrix_coefficients;
   std::deque<int64_t> references;
   uint32_t a_channels, a_bitdepth;
   float a_samplerate;
@@ -272,7 +272,7 @@ struct qtmp4_demuxer_c {
 
   std::string language;
 
-  debugging_option_c m_debug_tables, m_debug_frame_rate, m_debug_headers, m_debug_editlists;
+  debugging_option_c m_debug_tables, m_debug_tables_full, m_debug_frame_rate, m_debug_headers, m_debug_editlists, m_debug_indexes, m_debug_indexes_full;
 
   qtmp4_demuxer_c(qtmp4_reader_c &reader)
     : m_reader(reader)
@@ -284,7 +284,6 @@ struct qtmp4_demuxer_c {
     , time_scale{1}
     , duration{0}
     , global_duration{0}
-    , constant_editlist_offset_ns{0}
     , num_frames_from_trun{}
     , sample_size{0}
     , esds_parsed{false}
@@ -292,6 +291,9 @@ struct qtmp4_demuxer_c {
     , v_width{0}
     , v_height{0}
     , v_bitdepth{0}
+    , v_colour_primaries{2}
+    , v_colour_transfer_characteristics{2}
+    , v_colour_matrix_coefficients{2}
     , a_channels{0}
     , a_bitdepth{0}
     , a_samplerate{0.0}
@@ -301,10 +303,13 @@ struct qtmp4_demuxer_c {
     , a_aac_config_parsed{false}
     , warning_printed{false}
     , ptzr{-1}
-    , m_debug_tables{          "qtmp4_full|qtmp4_tables"}
+    , m_debug_tables{            "qtmp4_full|qtmp4_tables|qtmp4_tables_full"}
+    , m_debug_tables_full{                               "qtmp4_tables_full"}
     , m_debug_frame_rate{"qtmp4|qtmp4_full|qtmp4_frame_rate"}
     , m_debug_headers{   "qtmp4|qtmp4_full|qtmp4_headers"}
     , m_debug_editlists{ "qtmp4|qtmp4_full|qtmp4_editlists"}
+    , m_debug_indexes{         "qtmp4_full|qtmp4_indexes|qtmp4_indexes_full"}
+    , m_debug_indexes_full{                             "qtmp4_indexes_full"}
   {
     memset(&esds, 0, sizeof(esds_t));
   }
@@ -313,12 +318,12 @@ struct qtmp4_demuxer_c {
   }
 
   void calculate_frame_rate();
-  int64_t to_nsecs(int64_t value);
+  int64_t to_nsecs(int64_t value, boost::optional<int64_t> time_scale_to_use = boost::none);
   void calculate_timecodes();
   void adjust_timecodes(int64_t delta);
 
   bool update_tables();
-  void update_editlist_table();
+  void apply_edit_list();
 
   void build_index();
 
@@ -334,6 +339,7 @@ struct qtmp4_demuxer_c {
   void handle_audio_stsd_atom(uint64_t atom_size, int level);
   void handle_video_stsd_atom(uint64_t atom_size, int level);
   void handle_subtitles_stsd_atom(uint64_t atom_size, int level);
+  void handle_colr_atom(memory_cptr const &atom_content, int level);
 
   void parse_audio_header_priv_atoms(uint64_t atom_size, int level);
   void parse_video_header_priv_atoms(uint64_t atom_size, int level);
@@ -359,14 +365,17 @@ struct qtmp4_demuxer_c {
   void derive_track_params_from_mp3_audio_bitstream();
 
   void set_packetizer_display_dimensions();
+  void set_packetizer_colour_properties();
 
-  int64_t min_timecode() const;
+  boost::optional<int64_t> min_timecode() const;
 
   void determine_codec();
 
 private:
   void build_index_chunk_mode();
   void build_index_constant_sample_size_mode();
+  void dump_index_entries(std::string const &message) const;
+  void mark_key_frames_from_key_frame_table();
   void mark_open_gop_random_access_points_as_key_frames();
 
   void calculate_timecodes_constant_sample_size();
@@ -447,7 +456,7 @@ private:
 
   bool m_timecodes_calculated;
 
-  debugging_option_c m_debug_chapters, m_debug_headers, m_debug_tables, m_debug_interleaving, m_debug_resync;
+  debugging_option_c m_debug_chapters, m_debug_headers, m_debug_tables, m_debug_tables_full, m_debug_interleaving, m_debug_resync;
 
   friend class qtmp4_demuxer_c;
 
@@ -473,20 +482,21 @@ protected:
   virtual void parse_headers();
   virtual void verify_track_parameters_and_update_indexes();
   virtual void calculate_timecodes();
+  virtual boost::optional<int64_t> calculate_global_min_timecode() const;
   virtual qt_atom_t read_atom(mm_io_c *read_from = nullptr, bool exit_on_error = true);
   virtual bool resync_to_top_level_atom(uint64_t start_pos);
   virtual void parse_itunsmpb(std::string data);
 
   virtual void handle_cmov_atom(qt_atom_t parent, int level);
   virtual void handle_cmvd_atom(qt_atom_t parent, int level);
-  virtual void handle_ctts_atom(qtmp4_demuxer_cptr &new_dmx, qt_atom_t parent, int level);
-  virtual void handle_sgpd_atom(qtmp4_demuxer_cptr &new_dmx, qt_atom_t parent, int level);
-  virtual void handle_sbgp_atom(qtmp4_demuxer_cptr &new_dmx, qt_atom_t parent, int level);
+  virtual void handle_ctts_atom(qtmp4_demuxer_c &new_dmx, qt_atom_t parent, int level);
+  virtual void handle_sgpd_atom(qtmp4_demuxer_c &new_dmx, qt_atom_t parent, int level);
+  virtual void handle_sbgp_atom(qtmp4_demuxer_c &new_dmx, qt_atom_t parent, int level);
   virtual void handle_dcom_atom(qt_atom_t parent, int level);
-  virtual void handle_hdlr_atom(qtmp4_demuxer_cptr &new_dmx, qt_atom_t parent, int level);
-  virtual void handle_mdhd_atom(qtmp4_demuxer_cptr &new_dmx, qt_atom_t parent, int level);
-  virtual void handle_mdia_atom(qtmp4_demuxer_cptr &new_dmx, qt_atom_t parent, int level);
-  virtual void handle_minf_atom(qtmp4_demuxer_cptr &new_dmx, qt_atom_t parent, int level);
+  virtual void handle_hdlr_atom(qtmp4_demuxer_c &new_dmx, qt_atom_t parent, int level);
+  virtual void handle_mdhd_atom(qtmp4_demuxer_c &new_dmx, qt_atom_t parent, int level);
+  virtual void handle_mdia_atom(qtmp4_demuxer_c &new_dmx, qt_atom_t parent, int level);
+  virtual void handle_minf_atom(qtmp4_demuxer_c &new_dmx, qt_atom_t parent, int level);
   virtual void handle_moov_atom(qt_atom_t parent, int level);
   virtual void handle_mvhd_atom(qt_atom_t parent, int level);
   virtual void handle_udta_atom(qt_atom_t parent, int level);
@@ -500,42 +510,42 @@ protected:
   virtual void handle_traf_atom(qt_atom_t parent, int level);
   virtual void handle_tfhd_atom(qt_atom_t parent, int level);
   virtual void handle_trun_atom(qt_atom_t parent, int level);
-  virtual void handle_stbl_atom(qtmp4_demuxer_cptr &new_dmx, qt_atom_t parent, int level);
-  virtual void handle_stco_atom(qtmp4_demuxer_cptr &new_dmx, qt_atom_t parent, int level);
-  virtual void handle_co64_atom(qtmp4_demuxer_cptr &new_dmx, qt_atom_t parent, int level);
-  virtual void handle_stsc_atom(qtmp4_demuxer_cptr &new_dmx, qt_atom_t parent, int level);
-  virtual void handle_stsd_atom(qtmp4_demuxer_cptr &new_dmx, qt_atom_t parent, int level);
-  virtual void handle_stss_atom(qtmp4_demuxer_cptr &new_dmx, qt_atom_t parent, int level);
-  virtual void handle_stsz_atom(qtmp4_demuxer_cptr &new_dmx, qt_atom_t parent, int level);
-  virtual void handle_sttd_atom(qtmp4_demuxer_cptr &new_dmx, qt_atom_t parent, int level);
-  virtual void handle_stts_atom(qtmp4_demuxer_cptr &new_dmx, qt_atom_t parent, int level);
-  virtual void handle_tkhd_atom(qtmp4_demuxer_cptr &new_dmx, qt_atom_t parent, int level);
-  virtual void handle_trak_atom(qtmp4_demuxer_cptr &new_dmx, qt_atom_t parent, int level);
-  virtual void handle_edts_atom(qtmp4_demuxer_cptr &new_dmx, qt_atom_t parent, int level);
-  virtual void handle_elst_atom(qtmp4_demuxer_cptr &new_dmx, qt_atom_t parent, int level);
-  virtual void handle_tref_atom(qtmp4_demuxer_cptr &new_dmx, qt_atom_t parent, int level);
+  virtual void handle_stbl_atom(qtmp4_demuxer_c &new_dmx, qt_atom_t parent, int level);
+  virtual void handle_stco_atom(qtmp4_demuxer_c &new_dmx, qt_atom_t parent, int level);
+  virtual void handle_co64_atom(qtmp4_demuxer_c &new_dmx, qt_atom_t parent, int level);
+  virtual void handle_stsc_atom(qtmp4_demuxer_c &new_dmx, qt_atom_t parent, int level);
+  virtual void handle_stsd_atom(qtmp4_demuxer_c &new_dmx, qt_atom_t parent, int level);
+  virtual void handle_stss_atom(qtmp4_demuxer_c &new_dmx, qt_atom_t parent, int level);
+  virtual void handle_stsz_atom(qtmp4_demuxer_c &new_dmx, qt_atom_t parent, int level);
+  virtual void handle_sttd_atom(qtmp4_demuxer_c &new_dmx, qt_atom_t parent, int level);
+  virtual void handle_stts_atom(qtmp4_demuxer_c &new_dmx, qt_atom_t parent, int level);
+  virtual void handle_tkhd_atom(qtmp4_demuxer_c &new_dmx, qt_atom_t parent, int level);
+  virtual void handle_trak_atom(qtmp4_demuxer_c &new_dmx, qt_atom_t parent, int level);
+  virtual void handle_edts_atom(qtmp4_demuxer_c &new_dmx, qt_atom_t parent, int level);
+  virtual void handle_elst_atom(qtmp4_demuxer_c &new_dmx, qt_atom_t parent, int level);
+  virtual void handle_tref_atom(qtmp4_demuxer_c &new_dmx, qt_atom_t parent, int level);
 
-  virtual memory_cptr create_bitmap_info_header(qtmp4_demuxer_cptr &dmx, const char *fourcc, size_t extra_size = 0, const void *extra_data = nullptr);
+  virtual memory_cptr create_bitmap_info_header(qtmp4_demuxer_c &dmx, const char *fourcc, size_t extra_size = 0, const void *extra_data = nullptr);
 
-  virtual void create_audio_packetizer_aac(qtmp4_demuxer_cptr &dmx);
-  virtual bool create_audio_packetizer_ac3(qtmp4_demuxer_cptr &dmx);
-  virtual bool create_audio_packetizer_alac(qtmp4_demuxer_cptr &dmx);
-  virtual bool create_audio_packetizer_dts(qtmp4_demuxer_cptr &dmx);
-  virtual void create_audio_packetizer_mp3(qtmp4_demuxer_cptr &dmx);
-  virtual void create_audio_packetizer_passthrough(qtmp4_demuxer_cptr &dmx);
-  virtual void create_audio_packetizer_pcm(qtmp4_demuxer_cptr &dmx);
+  virtual void create_audio_packetizer_aac(qtmp4_demuxer_c &dmx);
+  virtual bool create_audio_packetizer_ac3(qtmp4_demuxer_c &dmx);
+  virtual bool create_audio_packetizer_alac(qtmp4_demuxer_c &dmx);
+  virtual bool create_audio_packetizer_dts(qtmp4_demuxer_c &dmx);
+  virtual void create_audio_packetizer_mp3(qtmp4_demuxer_c &dmx);
+  virtual void create_audio_packetizer_passthrough(qtmp4_demuxer_c &dmx);
+  virtual void create_audio_packetizer_pcm(qtmp4_demuxer_c &dmx);
 
-  virtual void create_video_packetizer_mpeg1_2(qtmp4_demuxer_cptr &dmx);
-  virtual void create_video_packetizer_mpeg4_p10(qtmp4_demuxer_cptr &dmx);
-  virtual void create_video_packetizer_mpeg4_p2(qtmp4_demuxer_cptr &dmx);
-  virtual void create_video_packetizer_mpegh_p2(qtmp4_demuxer_cptr &dmx);
-  virtual void create_video_packetizer_standard(qtmp4_demuxer_cptr &dmx);
-  virtual void create_video_packetizer_svq1(qtmp4_demuxer_cptr &dmx);
-  virtual void create_video_packetizer_prores(qtmp4_demuxer_cptr &dmx);
+  virtual void create_video_packetizer_mpeg1_2(qtmp4_demuxer_c &dmx);
+  virtual void create_video_packetizer_mpeg4_p10(qtmp4_demuxer_c &dmx);
+  virtual void create_video_packetizer_mpeg4_p2(qtmp4_demuxer_c &dmx);
+  virtual void create_video_packetizer_mpegh_p2(qtmp4_demuxer_c &dmx);
+  virtual void create_video_packetizer_standard(qtmp4_demuxer_c &dmx);
+  virtual void create_video_packetizer_svq1(qtmp4_demuxer_c &dmx);
+  virtual void create_video_packetizer_prores(qtmp4_demuxer_c &dmx);
 
-  virtual void create_subtitles_packetizer_vobsub(qtmp4_demuxer_cptr &dmx);
+  virtual void create_subtitles_packetizer_vobsub(qtmp4_demuxer_c &dmx);
 
-  virtual void handle_audio_encoder_delay(qtmp4_demuxer_cptr &dmx);
+  virtual void handle_audio_encoder_delay(qtmp4_demuxer_c &dmx);
 
   virtual std::string decode_and_verify_language(uint16_t coded_language);
   virtual void read_chapter_track();
