@@ -134,13 +134,14 @@ generic_reader_c::read_all() {
 bool
 generic_reader_c::demuxing_requested(char type,
                                      int64_t id,
-                                     std::string const &language) {
-  item_selector_c<bool> *tracks = 'v' == type ? &m_ti.m_vtracks
-                                : 'a' == type ? &m_ti.m_atracks
-                                : 's' == type ? &m_ti.m_stracks
-                                : 'b' == type ? &m_ti.m_btracks
-                                : 'T' == type ? &m_ti.m_track_tags
-                                :               nullptr;
+                                     boost::optional<std::string> const &language)
+  const {
+  auto const *tracks = 'v' == type ? &m_ti.m_vtracks
+                     : 'a' == type ? &m_ti.m_atracks
+                     : 's' == type ? &m_ti.m_stracks
+                     : 'b' == type ? &m_ti.m_btracks
+                     : 'T' == type ? &m_ti.m_track_tags
+                     :               nullptr;
 
   if (!tracks)
     mxerror(boost::format("generic_reader_c::demuxing_requested: %2%") % (boost::format(Y("Invalid track type %1%.")) % type));
