@@ -17,9 +17,11 @@
 #include "common/unique_numbers.h"
 #include "common/version.h"
 #include "mkvtoolnix-gui/app.h"
+#include "mkvtoolnix-gui/jobs/program_runner.h"
 #include "mkvtoolnix-gui/main_window/main_window.h"
 #include "mkvtoolnix-gui/merge/tool.h"
 #include "mkvtoolnix-gui/util/container.h"
+#include "mkvtoolnix-gui/util/media_player.h"
 #include "mkvtoolnix-gui/util/network_access_manager.h"
 #include "mkvtoolnix-gui/util/process.h"
 #include "mkvtoolnix-gui/util/settings.h"
@@ -39,6 +41,8 @@ class AppPrivate {
   std::unique_ptr<QTranslator> m_currentTranslator;
   std::unique_ptr<GuiCliParser> m_cliParser;
   std::unique_ptr<QLocalServer> m_instanceCommunicator;
+  std::unique_ptr<Jobs::ProgramRunner> m_programRunner;
+  std::unique_ptr<Util::MediaPlayer> m_mediaPlayer;
   Util::NetworkAccessManager *m_networkAccessManager{new Util::NetworkAccessManager{}};
   QThread m_networkAccessManagerThread;
   bool m_otherInstanceRunning{};
@@ -162,6 +166,34 @@ App::instance() {
   return static_cast<App *>(QApplication::instance());
 }
 
+Jobs::ProgramRunner &
+App::setupProgramRunner() {
+  Q_D(App);
+
+  if (!d->m_programRunner)
+    d->m_programRunner = Jobs::ProgramRunner::create();
+  return *d->m_programRunner;
+}
+
+Jobs::ProgramRunner &
+App::programRunner() {
+  return instance()->setupProgramRunner();
+}
+
+Util::MediaPlayer &
+App::setupMediaPlayer() {
+  Q_D(App);
+
+  if (!d->m_mediaPlayer)
+    d->m_mediaPlayer.reset(new Util::MediaPlayer);
+  return *d->m_mediaPlayer;
+}
+
+Util::MediaPlayer &
+App::mediaPlayer() {
+  return instance()->setupMediaPlayer();
+}
+
 void
 App::retranslateUi() {
   if (MainWindow::get())
@@ -426,17 +458,17 @@ void
 App::handleCommandLineArgumentsLocally() {
   Q_D(App);
 
-  if (!d->m_cliParser->m_configFiles.isEmpty())
-    emit openConfigFilesRequested(d->m_cliParser->m_configFiles);
+  if (!d->m_cliParser->configFiles().isEmpty())
+    emit openConfigFilesRequested(d->m_cliParser->configFiles());
 
-  if (!d->m_cliParser->m_addToMerge.isEmpty())
-    emit addingFilesToMergeRequested(d->m_cliParser->m_addToMerge);
+  if (!d->m_cliParser->addToMerge().isEmpty())
+    emit addingFilesToMergeRequested(d->m_cliParser->addToMerge());
 
-  if (!d->m_cliParser->m_editChapters.isEmpty())
-    emit editingChaptersRequested(d->m_cliParser->m_editChapters);
+  if (!d->m_cliParser->editChapters().isEmpty())
+    emit editingChaptersRequested(d->m_cliParser->editChapters());
 
-  if (!d->m_cliParser->m_editHeaders.isEmpty())
-    emit editingHeadersRequested(d->m_cliParser->m_editHeaders);
+  if (!d->m_cliParser->editHeaders().isEmpty())
+    emit editingHeadersRequested(d->m_cliParser->editHeaders());
 }
 
 void
