@@ -302,12 +302,14 @@ Tab::setupInputControls() {
   m_startMuxingMenu->addAction(m_startMuxingLeaveAsIs);
   m_startMuxingMenu->addAction(m_startMuxingCreateNewSettings);
   m_startMuxingMenu->addAction(m_startMuxingRemoveInputFiles);
+  m_startMuxingMenu->addAction(m_startMuxingCloseSettings);
   ui->startMuxing->setMenu(m_startMuxingMenu);
 
   // "add to job queue" menu
   m_addToJobQueueMenu->addAction(m_addToJobQueueLeaveAsIs);
   m_addToJobQueueMenu->addAction(m_addToJobQueueCreateNewSettings);
   m_addToJobQueueMenu->addAction(m_addToJobQueueRemoveInputFiles);
+  m_addToJobQueueMenu->addAction(m_addToJobQueueCloseSettings);
   ui->addToJobQueue->setMenu(m_addToJobQueueMenu);
 
   m_addFilesAction2->setIcon(QIcon{Q(":/icons/16x16/list-add.png")});
@@ -393,10 +395,12 @@ Tab::setupInputControls() {
 
   connect(m_startMuxingLeaveAsIs,           &QAction::triggered,                                                                              this,                     [=]() { addToJobQueue(true,  CMSAction::None); });
   connect(m_startMuxingCreateNewSettings,   &QAction::triggered,                                                                              this,                     [=]() { addToJobQueue(true,  CMSAction::NewSettings); });
+  connect(m_startMuxingCloseSettings,       &QAction::triggered,                                                                              this,                     [=]() { addToJobQueue(true,  CMSAction::CloseSettings); });
   connect(m_startMuxingRemoveInputFiles,    &QAction::triggered,                                                                              this,                     [=]() { addToJobQueue(true,  CMSAction::RemoveInputFiles); });
 
   connect(m_addToJobQueueLeaveAsIs,         &QAction::triggered,                                                                              this,                     [=]() { addToJobQueue(false, CMSAction::None); });
   connect(m_addToJobQueueCreateNewSettings, &QAction::triggered,                                                                              this,                     [=]() { addToJobQueue(false, CMSAction::NewSettings); });
+  connect(m_addToJobQueueCloseSettings,     &QAction::triggered,                                                                              this,                     [=]() { addToJobQueue(false, CMSAction::CloseSettings); });
   connect(m_addToJobQueueRemoveInputFiles,  &QAction::triggered,                                                                              this,                     [=]() { addToJobQueue(false, CMSAction::RemoveInputFiles); });
 
   connect(m_addFilesMenu,                   &QMenu::aboutToShow,                                                                              this,                     &Tab::enableFilesActions);
@@ -455,9 +459,11 @@ Tab::setupInputToolTips() {
                    .arg(QYH("The value can be given either as a floating point number (e.g. 12.345) or a fraction of integer values (e.g. 123/456)."))
                    .arg(QYH("This works well for video and subtitle tracks but should not be used with audio tracks.")));
   Util::setToolTip(ui->defaultDuration,
-                   Q("%1 %2")
+                   Q("%1 %2 %3 %4")
                    .arg(QY("Forces the default duration or number of frames per second for a track."))
-                   .arg(QY("The value can be given either as a floating point number (e.g. 12.345) or a fraction of integer values (e.g. 123/456).")));
+                   .arg(QY("The value can be given either as a floating point number (e.g. 12.345) or a fraction of integer values (e.g. 123/456)."))
+                   .arg(QY("You can specify one of the units 's', 'ms', 'us', 'ns', 'fps', 'i' or 'p'."))
+                   .arg(QY("If no unit is given, 'fps' will be used.")));
   Util::setToolTip(ui->fixBitstreamTimingInfo,
                    Q("%1 %2 %3")
                    .arg(QY("Normally mkvmerge does not change the timing information (frame/field rate) stored in the video bitstream."))
@@ -1357,10 +1363,12 @@ Tab::retranslateInputUI() {
 
   m_startMuxingLeaveAsIs->setText(QY("Afterwards &leave the settings as they are."));
   m_startMuxingCreateNewSettings->setText(QY("Afterwards create &new multiplex settings and close the current ones."));
+  m_startMuxingCloseSettings->setText(QY("Afterwards &close the current multiplex settings."));
   m_startMuxingRemoveInputFiles->setText(QY("Afterwards &remove all source files."));
 
   m_addToJobQueueLeaveAsIs->setText(QY("Afterwards &leave the settings as they are."));
   m_addToJobQueueCreateNewSettings->setText(QY("Afterwards create &new multiplex settings and close the current ones."));
+  m_addToJobQueueCloseSettings->setText(QY("Afterwards &close the current multiplex settings."));
   m_addToJobQueueRemoveInputFiles->setText(QY("Afterwards &remove all source files."));
 
   for (auto idx = 0u, end = stereo_mode_c::max_index(); idx <= end; ++idx)
