@@ -21,6 +21,17 @@ class Settings: public QObject {
   Q_ENUMS(RunProgramForEvent);
 
 public:
+  enum RunProgramType {
+    Min,
+    ExecuteProgram,
+    PlayAudioFile,
+    ShutDownComputer,
+    HibernateComputer,
+    SleepComputer,
+    Max,
+    Default = ExecuteProgram,
+  };
+
   enum RunProgramForEvent {
     RunNever                         = 0x00,
     RunAfterJobQueueFinishes         = 0x01,
@@ -69,6 +80,7 @@ public:
     None,
     NewSettings,
     RemoveInputFiles,
+    CloseSettings,
   };
 
   enum class MergeAddingAppendingFilesPolicy {
@@ -94,11 +106,21 @@ public:
 
   class RunProgramConfig {
   public:
+    RunProgramType m_type{RunProgramType::ExecuteProgram};
     bool m_active{true};
-    QStringList m_commandLine;
+    QString m_name;
     RunProgramForEvents m_forEvents{};
+    QStringList m_commandLine;
+    QString m_audioFile;
+    unsigned int m_volume{75};
 
     bool isValid() const;
+    QString validate() const;
+    QString name() const;
+
+  private:
+    QString nameForExternalProgram() const;
+    QString nameForPlayAudioFile() const;
   };
 
   using RunProgramConfigPtr  = std::shared_ptr<RunProgramConfig>;
@@ -106,7 +128,7 @@ public:
 
   QString m_defaultAudioTrackLanguage, m_defaultVideoTrackLanguage, m_defaultSubtitleTrackLanguage;
   SetDefaultLanguagePolicy m_whenToSetDefaultLanguage;
-  QString m_chapterNameTemplate, m_defaultChapterLanguage, m_defaultChapterCountry, m_defaultSubtitleCharset, m_defaultAdditionalMergeOptions;
+  QString m_chapterNameTemplate, m_defaultChapterLanguage, m_defaultChapterCountry, m_ceTextFileCharacterSet, m_defaultSubtitleCharset, m_defaultAdditionalMergeOptions;
   QStringList m_oftenUsedLanguages, m_oftenUsedCountries, m_oftenUsedCharacterSets;
   bool m_oftenUsedLanguagesOnly, m_oftenUsedCountriesOnly, m_oftenUsedCharacterSetsOnly;
   ProcessPriority m_priority;
@@ -178,6 +200,10 @@ protected:
   void saveSplitterSizes(QSettings &reg) const;
   void saveRunProgramConfigurations(QSettings &reg) const;
 
+  void addDefaultRunProgramConfigurations(QSettings &reg);
+  void addDefaultRunProgramConfigurationForType(QSettings &reg, RunProgramType type, std::function<void(RunProgramConfig &)> const &modifier = nullptr);
+  bool fixDefaultAudioFileNameBug();
+
 protected:
   static Settings s_settings;
 
